unit unitcomport;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is a serial port interface.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2009
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 * 
 * ***** END LICENSE BLOCK ***** *)

interface

uses
  Windows;

type
  TComPort = class
  private
    handle: THandle;
    bDCB: DCB;
  public
    constructor Create(name: string; baudrate: Cardinal);
    destructor Destroy(); override;
    function getMaxBaudrate(): Cardinal;
    function write(var buf; n: Cardinal): Cardinal;
    function read(var buf; n: Cardinal): Cardinal;
  end;

implementation

const
  fBinary = $0001;
  fParity = $0002;
  fOutxCtsFlow = $0004;
  fOutxDsrFlow = $0008;
  fDtrControlDisable = DTR_CONTROL_DISABLE shl 4;
  fDtrControlEnable = DTR_CONTROL_ENABLE shl 4;
  fDtrControlHandshake = DTR_CONTROL_HANDSHAKE shl 4;
  fDsrSensitivity = $0040;
  fTXContinueOnXoff = $0080;
  fOutX = $0100;
  fInX = $0200;
  fErrorChar = $0400;
  fNull = $0800;
  fRtsControlDisable = RTS_CONTROL_DISABLE shl 12;
  fRtsControlEnable = RTS_CONTROL_ENABLE shl 12;
  fRtsControlHandshake = RTS_CONTROL_HANDSHAKE shl 12;
  fRtsControlToggle = RTS_CONTROL_TOGGLE shl 12;
  fAbortOnError = $4000;

constructor TComPort.Create(name: string; baudrate: Cardinal);
begin
  inherited Create();
  handle := CreateFile(PChar(name), GENERIC_WRITE or GENERIC_READ, 0, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
  GetCommState(handle, bDCB);
  bDCB.BaudRate := baudrate;
  bDCB.ByteSize := 8;
  bDCB.StopBits := ONESTOPBIT;
  bDCB.Parity := NOPARITY;
  bDCB.Flags := fBinary or fDtrControlEnable or fTXContinueOnXoff or fRtsControlDisable;
  SetCommState(handle, bDCB);
  SetupComm(handle, 512, 512);
  EscapeCommFunction(handle, CLRDTR);
end;

destructor TComPort.Destroy();
begin
  CloseHandle(handle);
  inherited Destroy();
end;

function TComPort.getMaxBaudrate(): Cardinal;
var
  bCOMMPROP: COMMPROP;
begin
  GetCommProperties(handle, bCOMMPROP);
  Result := bCOMMPROP.dwMaxBaud;
end;

function TComPort.write(var buf; n: Cardinal): Cardinal;
begin
  WriteFile(handle, buf, n, Result, nil);
end;

function TComPort.read(var buf; n: Cardinal): Cardinal;
begin
  ReadFile(handle, buf, n, Result, nil);
end;

end.
