unit unitconfiguration;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the DMXLink configuration dialog.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2009
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 * 
 * ***** END LICENSE BLOCK ***** *)

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, unitportdetect;

const
  MAX_PORTS = 256;

type
  TformConfiguration = class(TForm)
    bOK: TButton;
    bCancel: TButton;
    gbSettings: TGroupBox;
    labelDevice: TLabel;
    labelInputInterval: TLabel;
    labelInputIntervalMs: TLabel;
    comboBoxDevice: TComboBox;
    checkBoxInputEnabled: TCheckBox;
    chOutputEnabled: TCheckBox;
    editInterval: TEdit;
    upDownInterval: TUpDown;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure bOKClick(Sender: TObject);
  private
    { Private declarations }
    ports: array[0..MAX_PORTS - 1] of TDetectedPort;
    portNumber: Integer;
  public
    { Public declarations }
    procedure setPortNumber(portNumber: Integer);
    function getPortNumber(): Integer;
  end;

var
  formConfiguration: TformConfiguration;

implementation

{$R *.dfm}

procedure TformConfiguration.FormCreate(Sender: TObject);
begin
  portNumber := -1;
end;

procedure TformConfiguration.setPortNumber(portNumber: Integer);
begin
  Self.portNumber := portNumber;
end;

function TformConfiguration.getPortNumber(): Integer;
begin
  Result := portNumber;
end;

procedure TformConfiguration.FormShow(Sender: TObject);
var
  i, index: Integer;
  instanceId, instanceIdSerial: PChar;
  serial, friendlyName: string;
begin
  getCOMPorts(ports);
  comboBoxDevice.Items.Clear();
  comboBoxDevice.Items.AddObject('(none)', TObject(0));
  index := 0;
  for i := 0 to MAX_PORTS - 1 do
    if ports[i].friendlyName <> '' then
    begin
      GetMem(instanceId, Length(ports[i].instanceId) + 1);
      try
        StrPCopy(instanceId, ports[i].instanceId);
        instanceIdSerial := StrRScan(instanceId, '\') + 1;
        serial := instanceIdSerial;
      finally
        FreeMem(instanceId);
      end;
      if Pos('DMXUNI', serial) > 0 then
        friendlyName := serial
      else
        friendlyName := ports[i].friendlyName;
      comboBoxDevice.Items.AddObject(friendlyName, TObject(i + 1));
      if i = portNumber then
        index := comboBoxDevice.Items.Count - 1;
    end;
  comboBoxDevice.ItemIndex := index;
end;

procedure TformConfiguration.bOKClick(Sender: TObject);
begin
  if comboBoxDevice.ItemIndex < comboBoxDevice.Items.Count then
    portNumber := Integer(comboBoxDevice.Items.Objects[comboBoxDevice.ItemIndex]) - 1;
end;

end.
