unit unitdmx4all;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is a software interface for the DMX4ALL hardware.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2009
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 * 
 * ***** END LICENSE BLOCK ***** *)

interface

uses
  Windows, SysUtils, unitcomport;

procedure dmxInit(portNumber: Integer);
procedure dmxFinish();
function dmxChannelOutput(channel: Word; value: Byte): Boolean;
function dmxChannelInput(channel: Word; var value: Byte): Boolean;
function dmxBlockOutput(startChannel: Word; values: PByte): Boolean;
function dmxBlockInput(startChannel: Word; values: PByte): Boolean;

var
  comPort: TComPort = nil;

implementation

procedure dmxInit(portNumber: Integer);
begin
  comPort := TComPort.Create('\\.\COM' + IntToStr(portNumber + 1), 115200);
end;

procedure dmxFinish();
begin
  if comPort <> nil then
    comPort.Free();
end;

function dmxChannelOutput(channel: Word; value: Byte): Boolean;
var
  comArrOut: array[0..7] of Byte;
  m: Integer;
  ack: Byte;
begin
  if comPort <> nil then
  begin
    comArrOut[0] := Ord('C');
    comArrOut[1] := Ord('0') + channel div 100;
    m := channel mod 100;
    comArrOut[2] := Ord('0') + m div 10;
    comArrOut[3] := Ord('0') + m mod 10;
    comArrOut[4] := Ord('L');
    comArrOut[5] := Ord('0') + value div 100;
    m := value mod 100;
    comArrOut[6] := Ord('0') + m div 10;
    comArrOut[7] := Ord('0') + m mod 10;
    comPort.write(comArrOut[0], 8);
    comPort.read(ack, 1);
    Result := ack = Ord('G');
  end
  else
    Result := False;
end;


function dmxChannelInput(channel: Word; var value: Byte): Boolean;
var
  comArrOut: array[0..4] of Byte;
  comArrIn: array[0..4] of Byte;
  m: Integer;
begin
  if comPort <> nil then
  begin
    comArrOut[0] := Ord('V');
    comArrOut[1] := Ord('0') + channel div 100;
    m := channel mod 100;
    comArrOut[2] := Ord('0') + m div 10;
    comArrOut[3] := Ord('0') + m mod 10;
    comArrOut[4] := Ord('?');
    comPort.write(comArrOut[0], 5);
    comArrIn[4] := 0;
    if (comPort.read(comArrIn[0], 4) = 4) and (comArrIn[3] = Ord('G')) then
    begin
      value := 100 * (comArrIn[0] - Ord('0')) + 10 * (comArrIn[1] - Ord('0')) + (comArrIn[2] - Ord('0'));
      Result := True;
    end
    else
    begin
      value := 0;
      Result := False;
    end;
  end
  else
    Result := False;
end;

function dmxBlockOutput(startChannel: Word; values: PByte): Boolean;
var
  comArrOut: array[0..3 + 128] of Byte;
  i: Integer;
  ack: Byte;
begin
  if comPort <> nil then
  begin
    comArrOut[0] := $FF;
    comArrOut[1] := LOBYTE(startChannel);
    comArrOut[2] := HIBYTE(startChannel);
    comArrOut[3] := 128;
    for i := 0 to 128 - 1 do
      comArrOut[4 + i] := PByte(Integer(values) + i)^;
    comPort.write(comArrOut[0], 4 + 128);
    comPort.read(ack, 1);
    Result := ack = Ord('G');
  end
  else
    Result := False;
end;

function dmxBlockInput(startChannel: Word; values: PByte): Boolean;
var
  comArrOut: array[0..3] of Byte;
  comArrIn: array[0..127 + 1] of Byte;
  i: Integer;
begin
  if comPort <> nil then
  begin
    comArrOut[0] := $FC;
    comArrOut[1] := LOBYTE(startChannel);
    comArrOut[2] := HIBYTE(startChannel);
    comArrOut[3] := 128;
    comPort.write(comArrOut[0], 4);
    comPort.read(comArrIn[0], 128 + 1);
    if comArrIn[128] = Ord('G') then
    begin
      for i := 0 to 128 - 1 do
        PByte(Integer(values) + i)^ := comArrIn[i];
      Result := True;
    end
    else
    begin
      for i := 0 to 128 - 1 do
        PByte(Integer(values) + i)^ := 0;
      Result := False;
    end;
  end
  else
    Result := False;
end;

end.
