unit unitdmxlink;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the DMXLink main window.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2009
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 * 
 * ***** END LICENSE BLOCK ***** *)

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, AppEvnts, ExtCtrls, Menus, ShellAPI, Grids, Registry;

const
  WM_NOTIFYICON = WM_USER + 333;

  nOutputs = 512; // excluding start code
  nInputs  = 256; // excluding start code

type
  TfDMXLink = class(TForm)
    aeEvents: TApplicationEvents;
    pmConfiguration: TPopupMenu;
    miReinitialize: TMenuItem;
    stringGridDMXOut: TStringGrid;
    stringGridDMXIn: TStringGrid;
    tRefreshInput: TTimer;
    menuItemConfiguration: TMenuItem;
    procedure initialize;
    procedure finalize;
    procedure aeEventsMessage(var Msg: tagMSG; var Handled: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure miReinitializeClick(Sender: TObject);
    procedure aeEventsMinimize(Sender: TObject);
    procedure aeEventsRestore(Sender: TObject);
    procedure tRefreshInputTimer(Sender: TObject);
    procedure menuItemConfigurationClick(Sender: TObject);
  private
    { Private declarations }
    tableOut: array[0..nOutputs - 1] of Byte;
    tableIn: array[0..nInputs - 1] of Byte;
    iconExists: Boolean;
    hMainIcon: THandle;
    bNotifyIconData: NOTIFYICONDATA;
    portNumber: Integer;
    procedure initTrayIcon();
    procedure finishTrayIcon();
    procedure addTrayIcon();
    procedure removeTrayIcon();
  protected
    procedure WMNOTIFYICON(var Message: TMessage); message WM_NOTIFYICON;
  public
    { Public declarations }
  end;

var
  fDMXLink: TfDMXLink;
  m_set_channel, m_get_channel: UINT;

implementation

{$R *.dfm}
{$R icons.res}

uses
  unitdmx4all, unitconfiguration;

procedure TfDMXLink.initialize();
begin
  if portNumber > -1 then
    dmxInit(portNumber);
end;

procedure TfDMXLink.finalize();
begin
  if portNumber > -1 then
    dmxFinish();
end;

procedure TfDMXLink.aeEventsMessage(var Msg: tagMSG; var Handled: Boolean);
var
  channel: Word;
  value: Byte;
begin
  if Msg.message = m_set_channel then
  begin
    channel := MakeWord(HIBYTE(LOWORD(Msg.lParam)), LOBYTE(HIWORD(Msg.lParam)));
    value := LOBYTE(LOWORD(Msg.lParam));
    tableOut[channel] := value;
  end;
end;

function EnumWindowsProc(hwnd: HWND; lParam: LPARAM): Boolean; stdcall;
var
  bPChar: PChar;
begin
  GetMem(bPChar, 20);
  try
    if GetClassName(hwnd, bPChar, 19) = 15 then
      if bPChar = 'TLightningDMXIn' then
        PostMessage(hwnd, m_get_channel, 0, lParam);
  finally
    FreeMem(bPChar);
  end;
  Result := True;
end;

procedure TfDMXLink.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  // fill up stringgrid caption cells.
  for i := 0 to nOutputs do
    tableOut[i] := 0;
  for i := 0 to nInputs do
    tableIn[i] := 0;
  for i := 0 to stringGridDMXOut.ColCount - 1 do
    stringGridDMXOut.Cells[i + 1, 0] := '+' + IntToStr(i);
  for i := 0 to (512 div (stringGridDMXOut.ColCount - 1)) do
    stringGridDMXOut.Cells[0, i + 1] := IntToStr(i * (stringGridDMXOut.ColCount - 1) + 1);
  for i := 0 to stringGridDMXIn.ColCount - 1 do
    stringGridDMXIn.Cells[i + 1, 0] := '+' + IntToStr(i);
  for i := 0 to (512 div (stringGridDMXIn.ColCount - 1)) do
    stringGridDMXIn.Cells[0, i + 1] := IntToStr(i * (stringGridDMXIn.ColCount - 1) + 1);
  portNumber := -1;
  with TRegistry.Create(KEY_READ) do
  begin
    try
      if OpenKey('SOFTWARE\Lightning\DMXLink', False) then
      begin
        try
          if ValueExists('dmx4all.port') then portNumber := ReadInteger('dmx4all.port');
        finally
          CloseKey();
        end;
      end;
    finally
      Free();
    end;
  end;
  m_set_channel := RegisterWindowMessage('DMXLINK');
  m_get_channel := RegisterWindowMessage('DMXLINK_GET_CHANNEL');
  initialize();
  initTrayIcon();
end;

procedure TfDMXLink.FormDestroy(Sender: TObject);
begin
  finalize();
  finishTrayIcon();
end;

procedure TfDMXLink.miReinitializeClick(Sender: TObject);
begin
  finalize();
  initialize();
end;

procedure TfDMXLink.WMNOTIFYICON(var Message: TMessage);
begin
  if Message.lParam = WM_LBUTTONUP then
  begin
    Show();
    Application.Restore();
    Application.BringToFront();
    SetFocus();
  end;
end;

procedure TfDMXLink.aeEventsMinimize(Sender: TObject);
begin
  ShowWindow(Application.Handle, SW_HIDE);
  addTrayIcon();
end;

procedure TfDMXLink.aeEventsRestore(Sender: TObject);
begin
  removeTrayIcon();
end;

procedure TfDMXLink.initTrayIcon();
begin
  iconExists := False;
  hMainIcon := LoadIcon(hInstance, 'trayicon');
  with bNotifyIconData do
  begin
    cbSize              := SizeOf(bNotifyIconData);
    Wnd                 := Handle;
    uID                 := 0;
    uFlags              := NIF_ICON or NIF_MESSAGE or NIF_TIP;
    uCallbackMessage    := WM_NOTIFYICON;
    hIcon               := hMainIcon;
    StrPCopy(szTip, Application.Title);
  end;
  addTrayIcon();
end;

procedure TfDMXLink.finishTrayIcon();
begin
  removeTrayIcon();
end;

procedure TfDMXLink.addTrayIcon();
begin
  if not(iconExists) then
  begin
    Shell_NotifyIcon(NIM_ADD, @bNotifyIconData);
    iconExists := True;
  end;
end;

procedure TfDMXLink.removeTrayIcon();
begin
  if iconExists then
  begin
    Shell_NotifyIcon(NIM_DELETE, @bNotifyIconData);
    iconExists := False;
  end;
end;

procedure TfDMXLink.tRefreshInputTimer(Sender: TObject);
var
  i: Integer;
  s: string;
begin
  if stringGridDMXOut.Visible then
  begin
    for i := 0 to nOutputs - 1 do
    begin
      stringGridDMXOut.Cells[
        i mod (stringGridDMXOut.ColCount - 1) + 1,
        i div (stringGridDMXOut.ColCount - 1) + 1
      ] := IntToStr(tableOut[i]);
    end;
  end;
  if portNumber > -1 then
  begin
    dmxBlockOutput(0,   @tableOut[0]);
    dmxBlockOutput(128, @tableOut[128]);
    dmxBlockOutput(256, @tableOut[256]);
    dmxBlockOutput(384, @tableOut[384]);
    dmxBlockInput(0,   @tableIn[0]);
    dmxBlockInput(128, @tableIn[128]);
    //dmxBlockInput(256, @tableIn[256]);
    //dmxBlockInput(384, @tableIn[384]);
    begin
      for i := 0 to nInputs - 1 do
      begin
        s := IntToStr(tableIn[i]);
        if s <> stringGridDMXIn.Cells[i mod (stringGridDMXOut.ColCount - 1) + 1, i div (stringGridDMXOut.ColCount - 1) + 1] then
        begin
          stringGridDMXIn.Cells[
            i mod (stringGridDMXOut.ColCount - 1) + 1,
            i div (stringGridDMXOut.ColCount - 1) + 1
          ] := s;
          if i < 128 then
            EnumWindows(@EnumWindowsProc, MAKEWORD(tableIn[i], i));
        end;
      end;
    end;
  end;
end;

procedure TfDMXLink.menuItemConfigurationClick(Sender: TObject);
begin
  formConfiguration.setPortNumber(portNumber);
  if formConfiguration.ShowModal() = mrOk then
  begin
    finalize();
    portNumber := formConfiguration.getPortNumber();
    initialize();
  end;
end;

end.
